@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QQUICKITEM_P_H
 #define QQUICKITEM_P_H
@@ -59,6 +23,7 @@
 #include <QtQuick/private/qquickstate_p.h>
 #include <QtQuick/private/qquickpaletteproviderprivatebase_p.h>
 #include <QtQuick/private/qquickwindow_p.h>
+#include <QtCore/private/qproperty_p.h>
 
 #if QT_CONFIG(quick_shadereffect)
 #include <QtQuick/private/qquickshadereffectsource_p.h>
@@ -285,6 +250,7 @@ public:
     bool hasPointerHandlers() const;
     bool hasHoverHandlers() const;
     virtual void addPointerHandler(QQuickPointerHandler *h);
+    virtual void removePointerHandler(QQuickPointerHandler *h);
 
     // data property
     static void data_append(QQmlListProperty<QObject> *, QObject *);
@@ -330,6 +296,8 @@ public:
         ImplicitWidth = 0x100,
         ImplicitHeight = 0x200,
         Enabled = 0x400,
+        Focus = 0x800,
+        AllChanges = 0xFFFFFFFF
     };
 
     Q_DECLARE_FLAGS(ChangeTypes, ChangeType)
@@ -357,6 +325,32 @@ public:
         QQuickGeometryChange gTypes;  //NOTE: not used for ==
     };
 
+    // call QQuickItemChangeListener PMF
+    template <typename Fn, typename ...Args>
+    void notifyChangeListeners(QQuickItemPrivate::ChangeTypes changeTypes, Fn &&function, Args &&...args)
+    {
+        if (changeListeners.isEmpty())
+            return;
+
+        const auto listeners = changeListeners; // NOTE: intentional copy (QTBUG-54732)
+        for (const QQuickItemPrivate::ChangeListener &change : listeners) {
+            if (change.types & changeTypes)
+                (change.listener->*function)(args...);
+        }
+    }
+    // call functor
+    template <typename Fn>
+    void notifyChangeListeners(QQuickItemPrivate::ChangeTypes changeTypes, Fn &&function) {
+        if (changeListeners.isEmpty())
+            return;
+
+        const auto listeners = changeListeners; // NOTE: intentional copy (QTBUG-54732)
+        for (const QQuickItemPrivate::ChangeListener &change : listeners) {
+            if (change.types & changeTypes)
+                function(change);
+        }
+    }
+
     struct ExtraData {
         ExtraData();
 
@@ -388,6 +382,8 @@ public:
         QSGOpacityNode *opacityNode;
         QQuickDefaultClipNode *clipNode;
         QSGRootNode *rootNode;
+        // subsceneDeliveryAgent is set only if this item is the root of a subscene, not on all items within.
+        QQuickDeliveryAgent *subsceneDeliveryAgent = nullptr;
 
         // Mask contains() method
         QMetaMethod maskContains;
@@ -399,6 +395,7 @@ public:
         // extremely common to set acceptedMouseButtons to LeftButton, but very
         // rare to use any of the other buttons.
         Qt::MouseButtons acceptedMouseButtons;
+        Qt::MouseButtons acceptedMouseButtonsWithoutHandlers;
 
         QQuickItem::TransformOrigin origin:5;
         uint transparentForPositioner : 1;
@@ -415,6 +412,7 @@ public:
     QLazilyAllocated<ExtraData, ExtraDataTags> extra;
     // Contains mask
     QPointer<QObject> mask;
+    QPointer<QQuickItem> quickMask;
     // If the mask is an Item, inform it that it's being used as a mask (true) or is no longer being used (false)
     virtual void registerAsContainmentMask(QQuickItem * /* maskedItem */, bool /* set */) { }
 
@@ -437,9 +435,9 @@ public:
     inline QQuickItem::TransformOrigin origin() const;
 
     // Bit 0
-    quint32 flags:5;
-    bool widthValid:1;
-    bool heightValid:1;
+    quint32 flags:7;
+    bool widthValidFlag:1;
+    bool heightValidFlag:1;
     bool componentComplete:1;
     bool keepMouse:1;
     bool keepTouch:1;
@@ -447,9 +445,9 @@ public:
     bool smooth:1;
     bool antialiasing:1;
     bool focus:1;
+    // Bit 16
     bool activeFocus:1;
     bool notifiedFocus:1;
-    // Bit 16
     bool notifiedActiveFocus:1;
     bool filtersChildMouseEvents:1;
     bool explicitVisible:1;
@@ -464,9 +462,9 @@ public:
     bool inheritMirrorFromItem:1;
     bool isAccessible:1;
     bool culled:1;
+    // Bit 32
     bool hasCursor:1;
     bool subtreeCursorEnabled:1;
-    // Bit 32
     bool subtreeHoverEnabled:1;
     bool activeFocusOnTab:1;
     bool implicitAntialiasing:1;
@@ -480,6 +478,11 @@ public:
     bool replayingPressEvent:1;
     bool touchEnabled:1;
     bool hasCursorHandler:1;
+    // set true when this item does not expect events via a subscene delivery agent; false otherwise
+    bool maybeHasSubsceneDeliveryAgent:1;
+    // set true if this item or any child wants QQuickItemPrivate::transformChanged() to visit all children
+    // (e.g. when parent has ItemIsViewport and child has ItemObservesViewport)
+    bool subtreeTransformChangedEnabled:1;
 
     enum DirtyType {
         TransformOrigin         = 0x00000001,
@@ -558,13 +561,24 @@ public:
 
     static bool canAcceptTabFocus(QQuickItem *item);
 
-    qreal x;
-    qreal y;
-    qreal width;
-    qreal height;
+    void setX(qreal x) {q_func()->setX(x);}
+    void xChanged() {q_func()->xChanged();}
+    Q_OBJECT_COMPAT_PROPERTY(QQuickItemPrivate, qreal, x, &QQuickItemPrivate::setX, &QQuickItemPrivate::xChanged);
+    void setY(qreal y) {q_func()->setY(y);}
+    void yChanged() {q_func()->yChanged();}
+    Q_OBJECT_COMPAT_PROPERTY(QQuickItemPrivate, qreal, y, &QQuickItemPrivate::setY, &QQuickItemPrivate::yChanged);
+    void setWidth(qreal width) {q_func()->setWidth(width);}
+    void widthChanged() {q_func()->widthChanged();}
+    Q_OBJECT_COMPAT_PROPERTY(QQuickItemPrivate, qreal, width, &QQuickItemPrivate::setWidth, &QQuickItemPrivate::widthChanged);
+    void setHeight(qreal height) {q_func()->setHeight(height);}
+    void heightChanged() {q_func()->heightChanged();}
+    Q_OBJECT_COMPAT_PROPERTY(QQuickItemPrivate, qreal, height, &QQuickItemPrivate::setHeight, &QQuickItemPrivate::heightChanged);
     qreal implicitWidth;
     qreal implicitHeight;
 
+    bool widthValid() const { return widthValidFlag || (width.hasBinding() && !QQmlPropertyBinding::isUndefined(width.binding()) ); }
+    bool heightValid() const { return heightValidFlag || (height.hasBinding() && !QQmlPropertyBinding::isUndefined(height.binding()) ); }
+
     qreal baselineOffset;
 
     QList<QQuickTransform *> transforms;
@@ -600,13 +614,17 @@ public:
     }
 
     QPointF computeTransformOrigin() const;
-    virtual void transformChanged();
+    virtual bool transformChanged(QQuickItem *transformedItem);
 
     QPointF adjustedPosForTransform(const QPointF &centroid,
                                     const QPointF &startPos, const QVector2D &activeTranslatation,
                                     qreal startScale, qreal activeScale,
                                     qreal startRotation, qreal activeRotation);
 
+    QQuickDeliveryAgent *deliveryAgent();
+    QQuickDeliveryAgentPrivate *deliveryAgentPrivate();
+    QQuickDeliveryAgent *ensureSubsceneDeliveryAgent();
+
     void deliverKeyEvent(QKeyEvent *);
     bool filterKeyEvent(QKeyEvent *, bool post);
 #if QT_CONFIG(im)
@@ -615,7 +633,7 @@ public:
     void deliverShortcutOverrideEvent(QKeyEvent *);
 
     bool anyPointerHandlerWants(const QPointerEvent *event, const QEventPoint &point) const;
-    virtual bool handlePointerEvent(QPointerEvent *, bool avoidExclusiveGrabber = false);
+    virtual bool handlePointerEvent(QPointerEvent *, bool avoidGrabbers = false);
 
     virtual void setVisible(bool visible);
 
@@ -666,6 +684,7 @@ public:
 #endif
 
     virtual void updatePolish() { }
+    virtual void dumpItemTree(int indent) const;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QQuickItemPrivate::ExtraDataTags)
@@ -851,7 +870,7 @@ public:
     QQuickKeyEvent theKeyEvent;
 };
 
-class QQuickKeysAttached : public QObject, public QQuickItemKeyFilter
+class Q_QUICK_PRIVATE_EXPORT QQuickKeysAttached : public QObject, public QQuickItemKeyFilter
 {
     Q_OBJECT
     Q_DECLARE_PRIVATE(QQuickKeysAttached)
