@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QSGADAPTATIONLAYER_P_H
 #define QSGADAPTATIONLAYER_P_H
@@ -53,6 +17,7 @@
 
 #include <QtQuick/qsgnode.h>
 #include <QtQuick/qsgtexture.h>
+#include <QtQuick/qquickpainteditem.h>
 #include <QtCore/qobject.h>
 #include <QtCore/qrect.h>
 #include <QtGui/qbrush.h>
@@ -90,7 +55,7 @@ class QRhiTexture;
 class Q_QUICK_PRIVATE_EXPORT QSGNodeVisitorEx
 {
 public:
-    virtual ~QSGNodeVisitorEx() {}
+    virtual ~QSGNodeVisitorEx();
 
     // visit(...) returns true if the children are supposed to be
     // visisted and false if they're supposed to be skipped by the visitor.
@@ -128,6 +93,7 @@ class Q_QUICK_PRIVATE_EXPORT QSGVisitableNode : public QSGGeometryNode
 {
 public:
     QSGVisitableNode() { setFlag(IsVisitableNode); }
+    ~QSGVisitableNode() override;
 
     virtual void accept(QSGNodeVisitorEx *) = 0;
 };
@@ -135,6 +101,8 @@ public:
 class Q_QUICK_PRIVATE_EXPORT QSGInternalRectangleNode : public QSGVisitableNode
 {
 public:
+    ~QSGInternalRectangleNode() override;
+
     virtual void setRect(const QRectF &rect) = 0;
     virtual void setColor(const QColor &color) = 0;
     virtual void setPenColor(const QColor &color) = 0;
@@ -154,6 +122,8 @@ public:
 class Q_QUICK_PRIVATE_EXPORT QSGInternalImageNode : public QSGVisitableNode
 {
 public:
+    ~QSGInternalImageNode() override;
+
     virtual void setTargetRect(const QRectF &rect) = 0;
     virtual void setInnerTargetRect(const QRectF &rect) = 0;
     virtual void setInnerSourceRect(const QRectF &rect) = 0;
@@ -163,7 +133,7 @@ public:
     virtual void setSubSourceRect(const QRectF &rect) = 0;
     virtual void setTexture(QSGTexture *texture) = 0;
     virtual void setAntialiasing(bool antialiasing) { Q_UNUSED(antialiasing); }
-    virtual void setMirror(bool mirror) = 0;
+    virtual void setMirror(bool horizontally, bool vertically) = 0;
     virtual void setMipmapFiltering(QSGTexture::Filtering filtering) = 0;
     virtual void setFiltering(QSGTexture::Filtering filtering) = 0;
     virtual void setHorizontalWrapMode(QSGTexture::WrapMode wrapMode) = 0;
@@ -177,6 +147,7 @@ public:
 class Q_QUICK_PRIVATE_EXPORT QSGPainterNode : public QSGVisitableNode
 {
 public:
+    ~QSGPainterNode() override;
 
     virtual void setPreferredRenderTarget(QQuickPaintedItem::RenderTarget target) = 0;
     virtual void setSize(const QSize &size) = 0;
@@ -201,6 +172,13 @@ class Q_QUICK_EXPORT QSGLayer : public QSGDynamicTexture
 {
     Q_OBJECT
 public:
+    ~QSGLayer() override;
+
+    enum Format {
+        RGBA8 = 1,
+        RGBA16F,
+        RGBA32F
+    };
     virtual void setItem(QSGNode *item) = 0;
     virtual void setRect(const QRectF &rect) = 0;
     virtual void setSize(const QSize &size) = 0;
@@ -208,7 +186,7 @@ public:
     virtual QImage toImage() const = 0;
     virtual void setLive(bool live) = 0;
     virtual void setRecursive(bool recursive) = 0;
-    virtual void setFormat(uint format) = 0;
+    virtual void setFormat(Format format) = 0;
     virtual void setHasMipmaps(bool mipmap) = 0;
     virtual void setDevicePixelRatio(qreal ratio) = 0;
     virtual void setMirrorHorizontal(bool mirror) = 0;
@@ -230,6 +208,8 @@ protected:
 class Q_QUICK_PRIVATE_EXPORT QSGSpriteNode : public QSGVisitableNode
 {
 public:
+    ~QSGSpriteNode() override;
+
     virtual void setTexture(QSGTexture *texture) = 0;
     virtual void setTime(float time) = 0;
     virtual void setSourceA(const QPoint &source) = 0;
@@ -251,6 +231,8 @@ class Q_QUICK_PRIVATE_EXPORT QSGGuiThreadShaderEffectManager : public QObject
     Q_OBJECT
 
 public:
+    ~QSGGuiThreadShaderEffectManager() override;
+
     enum Status {
         Compiled,
         Uncompiled,
@@ -283,7 +265,6 @@ public:
         };
 
         QString name; // optional, f.ex. the filename, used for debugging purposes only
-        QByteArray blob; // source or bytecode (when not using QRhi)
         QShader rhiShader;
         Type type;
         QVector<Variable> variables;
@@ -311,6 +292,8 @@ class Q_QUICK_PRIVATE_EXPORT QSGShaderEffectNode : public QObject, public QSGVis
     Q_OBJECT
 
 public:
+    ~QSGShaderEffectNode() override;
+
     enum DirtyShaderFlag {
         DirtyShaders = 0x01,
         DirtyShaderConstant = 0x02,
@@ -385,6 +368,7 @@ public:
     };
 
     QSGGlyphNode() {}
+    ~QSGGlyphNode() override;
 
     virtual void setGlyphs(const QPointF &position, const QGlyphRun &glyphs) = 0;
     virtual void setColor(const QColor &color) = 0;
@@ -412,7 +396,7 @@ protected:
 class Q_QUICK_PRIVATE_EXPORT QSGDistanceFieldGlyphConsumer
 {
 public:
-    virtual ~QSGDistanceFieldGlyphConsumer() {}
+    virtual ~QSGDistanceFieldGlyphConsumer();
 
     virtual void invalidateGlyphs(const QVector<quint32> &glyphs) = 0;
     QIntrusiveListNode node;
@@ -464,9 +448,9 @@ public:
     {
         return pixelSize / baseFontSize();
     }
-    int distanceFieldRadius() const
+    qreal distanceFieldRadius() const
     {
-        return QT_DISTANCEFIELD_RADIUS(m_doubleGlyphResolution) / QT_DISTANCEFIELD_SCALE(m_doubleGlyphResolution);
+        return QT_DISTANCEFIELD_RADIUS(m_doubleGlyphResolution) / qreal(QT_DISTANCEFIELD_SCALE(m_doubleGlyphResolution));
     }
     int glyphCount() const { return m_glyphCount; }
     bool doubleGlyphResolution() const { return m_doubleGlyphResolution; }
@@ -490,6 +474,7 @@ public:
 
     virtual bool eightBitFormatIsAlphaSwizzled() const = 0;
     virtual bool screenSpaceDerivativesSupported() const = 0;
+    virtual bool isActive() const;
 
 protected:
     struct GlyphPosition {
